/*
** LEE_CACHE.C
**
** Lee registros preprocesados.
**
**
** (c) 1.997-2.000 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 08/Sep/99 Primera version de este modulo.
*/

#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <fcntl.h>
#include "lee_cache.h"
#include "version.h"

#ifndef _WIN32
#include <unistd.h>
#include <sys/fcntl.h>
#endif

#include "util.h"

/*
    $Id: lee_cache.c,v 1.8 2000/02/28 15:42:44 jcea Exp $
*/

static t_log orig;
static int fichero, bloqueo_handle;
#ifndef _WIN32
static struct flock bloqueo;
#endif
static int generando;
static char usuario[CAMPO_LONG];

#define lee_de_cache_int(a)	lee_de_cache(a,sizeof(int))
#define lee_de_cache_long(a)	lee_de_cache(a,sizeof(long))
#define lee_de_cache_long_long(a) lee_de_cache(a,sizeof(tipo_int64))
#define lee_de_cache(p,len)	lee_log_binario_cache(p,len)
#define almacena_en_cache_int(a)	almacena_en_cache(a,sizeof(int))
#define almacena_en_cache_long(a)	almacena_en_cache(a,sizeof(long))
#define almacena_en_cache_long_long(a) almacena_en_cache(a,sizeof(tipo_int64))


#define CACHE_ESCRITURA	(16*1024)
char cache[CACHE_ESCRITURA];
char *p_cache;


static void flush_cache_escritura(void)
{
  int len = p_cache - cache;

  if (len)
    write(fichero, cache, len);
  p_cache = cache;
}

static void almacena_en_cache(void *p, int len)
{
  if (p_cache - cache + len >= CACHE_ESCRITURA)
    flush_cache_escritura();
  memcpy(p_cache, p, len);
  p_cache += len;
}

void abre_log_detalle_cache(char *f_detalle, char *usr, t_log original)
{
  char buf[CAMPO_LONG];
  time_t timestamp, timestamp2;

  p_cache = cache;
  fichero = bloqueo_handle = -1;
  generando = 0;
  if (strlen(usr) > CAMPO_LONG - 13) {
    perror("Nombre de usuario demasiado largo");
    exit(-1);
  }
  else
    strcpy(usuario, usr);
  if (strlen(f_detalle) > CAMPO_LONG - 3) {
    perror("PATH demasiado largo");
    exit(-1);
  }

  strcpy(buf, f_detalle);
  strcat(buf, ".cache");
  bloqueo_handle = open(buf, O_WRONLY);
#ifndef _WIN32
  bloqueo.l_type = F_RDLCK;
  bloqueo.l_whence = 0;
  bloqueo.l_start = 0;
  bloqueo.l_len = 0;            /* Todo el fichero */
  fcntl(bloqueo_handle, F_SETLKW, &bloqueo);
  bloqueo.l_type = F_WRLCK;
#endif

  orig = original;
  orig.abre_log(f_detalle, "");
  timestamp = orig.timestamp();
  if (timestamp > time(NULL) - 60) {
#ifndef _WIN32
    fcntl(bloqueo_handle, F_SETLKW, &bloqueo);
#endif
    close(open(buf, O_WRONLY | O_TRUNC));
    close(bloqueo_handle);
    return;
  }
  orig.cierra_log();

  abre_log_cache(buf);
  lee_de_cache(&timestamp2, sizeof(time_t));
/*
** Regenera cache si fue generada por otra version
*/
  if (timestamp2 != libreria_timestamp())
    timestamp2 = timestamp + 1;
  else
    timestamp2 = lee_log_timestamp();
  if (timestamp2 - 60 < timestamp) {

#ifndef _WIN32
    fcntl(bloqueo_handle, F_SETLKW, &bloqueo);
#endif
    cierra_log_cache();
    orig.abre_log(f_detalle, "");
#ifndef _WIN32
    fcntl(bloqueo_handle, F_SETLKW, &bloqueo);
#endif
    fichero = open(buf, O_WRONLY | O_TRUNC);
    if (fichero == -1) {        /* No se pudo crear la cache */
      perror("Problemas para crear la cache");
      /* Lee del original */
      close(bloqueo_handle);
      return;
    }

    /*
       ** No grabamos el timestamp interno
       ** hasta que completamos la generacion
       ** de la cache.
     */
    timestamp = 0;
    almacena_en_cache(&timestamp, sizeof(time_t));
    generando = !0;             /* Vamos leyendo y generando */
    return;
  }

/* Tenemos cache valido */

  fichero = 0;
  return;
}

void cierra_log_detalle_cache(void)
{
  time_t timestamp;

  orig.cierra_log();
  if (fichero > 0) {
    /*
       ** Terminamos de escribir al final del fichero
     */
    flush_cache_escritura();
    /*
       ** Ponemos el timestamp cuando terminamos
       ** de generar la cache, por lo que pudiera pasar.
     */
    timestamp = libreria_timestamp();
    lseek(fichero, 0, SEEK_SET);
    almacena_en_cache(&timestamp, sizeof(time_t));
    flush_cache_escritura();
    close(fichero);
  }
  close(bloqueo_handle);
}

static void lee_cadena_de_cache(char *cadena)
{
  int len;

  lee_de_cache_int(&len);
  if (len)
    lee_de_cache(cadena, len);
  *(cadena + len) = '\0';
}

static log_estado lee_registro_de_cache(registro_log * registro)
{
  log_estado estado;

  inicializa_registro(registro);
  estado = lee_de_cache(&(registro->tipo), sizeof(registro->tipo));
  if (estado == LOG_OK) {
    lee_de_cache_long(&(registro->tiempo));
    lee_cadena_de_cache(registro->nombre);
    lee_cadena_de_cache(registro->sesion_id);
    lee_de_cache_long(&(registro->duracion));
    lee_de_cache_int(&(registro->causa));
    lee_cadena_de_cache(registro->causa_texto);
    lee_de_cache_long(&(registro->velocidad));
    lee_cadena_de_cache(registro->caller_id);
    lee_cadena_de_cache(registro->called_id);
    lee_cadena_de_cache(registro->ip);
    lee_de_cache_long_long(&(registro->bytes_tx));
    lee_de_cache_long_long(&(registro->bytes_rx));
    lee_de_cache_long_long(&(registro->paquetes_tx));
    lee_de_cache_long_long(&(registro->paquetes_rx));
  }
  return estado;
}

static void almacena_cadena_en_cache(char *cadena)
{
  int len = strlen(cadena);

  almacena_en_cache_int(&len);
  if (len)
    almacena_en_cache(cadena, len);
}

static void almacena_registro_en_cache(registro_log * registro)
{
  almacena_en_cache(&(registro->tipo), sizeof(registro->tipo));
  almacena_en_cache_long(&(registro->tiempo));
  almacena_cadena_en_cache(registro->nombre);
  almacena_cadena_en_cache(registro->sesion_id);
  almacena_en_cache_long(&(registro->duracion));
  almacena_en_cache_int(&(registro->causa));
  almacena_cadena_en_cache(registro->causa_texto);
  almacena_en_cache_long(&(registro->velocidad));
  almacena_cadena_en_cache(registro->caller_id);
  almacena_cadena_en_cache(registro->called_id);
  almacena_cadena_en_cache(registro->ip);
  almacena_en_cache_long_long(&(registro->bytes_tx));
  almacena_en_cache_long_long(&(registro->bytes_rx));
  almacena_en_cache_long_long(&(registro->paquetes_tx));
  almacena_en_cache_long_long(&(registro->paquetes_rx));
}

log_estado lee_registro_detalle_cache(registro_log * registro)
{
  log_estado estado;

bucle:

  if (generando) {
    estado = orig.lee_registro(registro);
    if (estado != LOG_OK)
      return estado;
    almacena_registro_en_cache(registro);
  }
  else if (fichero == -1)
    estado = orig.lee_registro(registro);
  else
    estado = lee_registro_de_cache(registro);

  if ((estado == LOG_OK) && (*usuario != '\0') && /* Queremos otro registro */
      (strcmp(registro->nombre, usuario)))
    goto bucle;

  return estado;
}

time_t timestamp_cache(void)
{
  return orig.timestamp();
}
