/*
** SALIDA_GRAFICO.C
**
** Genera GIF & PNG
**
**
** (c) 1.997-2.000 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 04/Ago/99 Primera version de este modulo.
*/

#include "salida_grafico.h"
#include "tiempo.h"
#include "festivos.h"

#if defined(GRAFICOS_GIF) || defined(GRAFICOS_PNG)

#include <gd.h>
#include <gdfonts.h>
#include <gdfontg.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
    $Id: salida_grafico.c,v 1.10 2000/02/10 11:23:33 jcea Exp $
*/

static char sesion_id[CAMPO_LONG];

static gdImagePtr im;
static int blanco, negro, verde1, verde2, rojo;

#define TAMANO_HORA	30
#define TAMANO_DIA	12
#define TAMANO_X	(TAMANO_HORA*24)

struct fallidos {
  int x;
  int y;
  struct fallidos *siguiente;
};

static struct fallidos *fallidos;

static char *usuario;
static int mes;
static int num_dias;
static char anho[5];

void inicializa_grafico(void)
{
  int cont, dia;
  int num_dia;
  static gdImagePtr im2;
  festivos_t dias_festivos;
  time_t dia1 = dia1_este_mes(tiempo_inicial);

  sesion_id[0] = '\0';

  im2 = gdImageCreate(TAMANO_DIA / 2, TAMANO_DIA / 2);
  blanco = gdImageColorAllocate(im2, 255, 255, 255); /* Fondo */
  rojo = gdImageColorAllocate(im2, 255, 0, 0);
  gdImageSetPixel(im2, 0, 0, rojo);
  gdImageSetPixel(im2, TAMANO_DIA / 4, TAMANO_DIA / 4, rojo);

  num_dias = num_dias_este_mes(dia1);

  im = gdImageCreate(TAMANO_X, TAMANO_DIA * num_dias + 4 + gdFontGiant->h);
  blanco = gdImageColorAllocate(im, 255, 255, 255); /* Fondo */
  negro = gdImageColorAllocate(im, 0, 0, 0);
  verde1 = gdImageColorAllocate(im, 0, 255, 0);
  verde2 = gdImageColorAllocate(im, 0, 255, 255);
  rojo = gdImageColorAllocate(im, 255, 0, 0);
  fallidos = NULL;
  usuario = NULL;

  gdImageSetTile(im, im2);

  num_dia = dia_de_la_semana(dia1);

  festivos(tiempo_inicial, &dias_festivos);
  for (cont = 1, dia = 0; dia < num_dias * TAMANO_DIA;
       dia += TAMANO_DIA, cont++) {
    gdImageLine(im, 0, dia, TAMANO_X - 1, dia, gdStyled);
    if ((num_dia == 5) || (num_dia == 6) || (dias_festivos[cont] != NULL))
      gdImageFilledRectangle(im, 0, dia, TAMANO_X - 1, dia + TAMANO_DIA - 1,
                             gdTiled);
    if (++num_dia == 7)
      num_dia = 0;
  }
  gdImageDestroy(im2);
}

static void calcula_posicion(char *buf, int *x, int *y)
{
  *y = ((buf[6] * 10 + buf[7]) - ('0' * 10 + '1')) * TAMANO_DIA;
  *x = ((((buf[8] * 10 + buf[9]) - ('0' * 10 + '0')) * 60
         + ((buf[10] * 10 + buf[11]) - ('0' * 10 + '0'))) * TAMANO_HORA) / 60;

}

void dibuja_grafico(char *f_detalle, char *nombre, t_log * log)
{
  registro_log reg;
  log_estado estado;
  time_t inicio, final;
  char buf[64];
  int x1, x2, y1, y2;
  int temp;
  struct fallidos *fallo;

  if (usuario == NULL) {
    tiempo2str(buf, tiempo_inicial);
    usuario = malloc(strlen(nombre) + 1);
    strcpy(usuario, nombre);
    mes = buf[4] * 10 + buf[5] - ('0' * 10 + '1');
    strncpy(anho, buf, 4);
    buf[4] = '\0';
  }

  log->abre_log(f_detalle, nombre);
  while (!0) {
    estado = log->lee_registro(&reg);
    if (estado != LOG_OK)
      break;
    if (reg.tipo != STOP)
      continue;

    if (!strcmp(sesion_id, reg.sesion_id))
      continue;

    strcpy(sesion_id, reg.sesion_id);

    final = reg.tiempo;
    inicio = final - reg.duracion;

    if (tiempo_inicial > final)
      continue;
    if (tiempo_final < inicio)
      continue;

    tiempo2str(buf, inicio);
    calcula_posicion(buf, &x1, &y1);

    if (inicio == final) {      /* Conexion fallida */
      fallo = malloc(sizeof(struct fallidos));
      if (fallo == NULL)
        exit(-1);
      fallo->siguiente = fallidos;
      fallidos = fallo;
      fallo->x = x1;
      fallo->y = y1;
      continue;
    }

    tiempo2str(buf, final);
    calcula_posicion(buf, &x2, &y2);
    if (y2 < y1) {              /* La conexion pilla en un cambio de mes */
      x1 = y1 = 0;              /* Ponemos como si empezase a las 00:00 */
    }
    while (y1 != y2) {
      gdImageFilledRectangle(im, x1, y1, TAMANO_X, y1 + TAMANO_DIA, verde1);
      y1 += TAMANO_DIA;
      x1 = 0;
    }
    gdImageFilledRectangle(im, x1, y1, x2, y1 + TAMANO_DIA, verde1);
    temp = verde1;
    verde1 = verde2;
    verde2 = temp;
  }
}

void envia_grafico(void)
{
  FILE *out;
  int dia, hora;
  int cont;
  int dash[4];
  char buf[1024];
  struct fallidos *fallo;
  static char *nombre_mes[] = {
    "Enero", "Febrero", "Marzo",
    "Abril", "Mayo", "Junio",
    "Julio", "Agosto", "Septiembre",
    "Octubre", "Noviembre", "Diciembre"
  };

  fallo = fallidos;
  while ((fallo = fallidos) != NULL) {
    gdImageLine(im, fallo->x, fallo->y, fallo->x, fallo->y + TAMANO_DIA,
                rojo);
    fallidos = fallo->siguiente;
    free(fallo);
  }

  if ((usuario != NULL) && (strlen(usuario) < 500)) {
    strcpy(buf, usuario);
    strcat(buf, " - ");
    strcat(buf, nombre_mes[mes]);
    strcat(buf, " ");
    strcat(buf, anho);
    cont = (TAMANO_X - strlen(buf) * gdFontGiant->w) / 2;
    gdImageString(im, gdFontGiant, cont, TAMANO_DIA * num_dias + 4, buf,
                  negro);
  }

  buf[2] = '\0';
  dash[0] = dash[1] = negro;
  dash[2] = dash[3] = gdTransparent;
  gdImageSetStyle(im, dash, 4);

  for (cont = 1, hora = TAMANO_HORA; hora < 24 * TAMANO_HORA;
       hora += TAMANO_HORA, cont++) {
    buf[0] = '0' + cont / 10;
    buf[1] = '0' + cont % 10;
    gdImageLine(im, hora, 0, hora, TAMANO_DIA * num_dias - 1, gdStyled);
    gdImageString(im, gdFontSmall, TAMANO_HORA / 3 + hora,
                  TAMANO_DIA * (num_dias - 1), buf, negro);
  }
  for (cont = 1, dia = 0; dia < num_dias * TAMANO_DIA;
       dia += TAMANO_DIA, cont++) {
    gdImageLine(im, 0, dia, TAMANO_X - 1, dia, gdStyled);
    buf[0] = '0' + cont / 10;
    buf[1] = '0' + cont % 10;
    gdImageString(im, gdFontSmall, TAMANO_HORA / 3, dia, buf, negro);
  }
  gdImageRectangle(im, 0, 0, TAMANO_X - 1, TAMANO_DIA * num_dias - 1, negro);

  out = fdopen(1, "ab");
#ifdef GRAFICOS_GIF
  fprintf(out, "Content-Type: image/gif\n\n");
  gdImageGif(im, out);          /* Salida estandar */
#endif
#ifdef GRAFICOS_PNG
  fprintf(out, "Content-Type: image/png\n\n");
  gdImagePng(im, out);          /* Salida estandar */
#endif
  gdImageDestroy(im);
  fclose(out);
}

#endif
