/*
** TIEMPO.C
**
** Rutinas de tratamiento de tiempo.
**
**
** (c) 1.997-2.000 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 08/Sep/99 Primera version de este modulo,
**           transferido desde config.c.
*/

#include "tiempo.h"
#include "conf.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/*
    $Id: tiempo.c,v 1.8 2000/01/04 13:06:48 jcea Exp $
*/

#define MAX_TIEMPO  1893452400ul /* 01/01/2030 */
time_t tiempo_inicial = 0;
time_t tiempo_final = MAX_TIEMPO;

/*
** Rutina de apoyo para la siguiente.
*/
static time_t str2tiempo2(char *tiempo)
{
  char buf[32];
  char c;
  struct tm tm;

/*
  strptime(tiempo,"%Y%m%d%H%M",&tm);
*/

  memset(buf, '\0', 31);
  strncpy(buf, tiempo, 30);
  c = buf[4];
  buf[4] = '\0';
  tm.tm_year = atoi(buf + 0) - 1900;
  buf[4] = c;
  c = buf[6];
  buf[6] = '\0';
  tm.tm_mon = atoi(buf + 4) - 1;
  buf[6] = c;
  c = buf[8];
  buf[8] = '\0';
  tm.tm_mday = atoi(buf + 6);
  buf[8] = c;
  c = buf[10];
  buf[10] = '\0';
  tm.tm_hour = atoi(buf + 8);
  buf[10] = c;
  c = buf[12];
  buf[12] = '\0';
  tm.tm_min = atoi(buf + 10);

  tm.tm_sec = 0;
  tm.tm_isdst = -1;

  return mktime(&tm);
}

/*
** El tiempo convertido es local, no UTC
** Gestiona ?correctamente? el Daylight Saving Time.
*/
time_t str2tiempo(char *tiempo)
{
  time_t t;
  char buf[64];

  t = str2tiempo2(tiempo);
  tiempo2str(buf, t);
  return t + (t - str2tiempo2(buf));
}

void tiempo2str(char *buf, time_t tiempo)
{
  struct tm *tm;

  tm = localtime(&tiempo);
  strftime(buf, CAMPO_LONG, "%Y%m%d%H%M", tm);
}

/*
** Esta rutina devuelve el dia de la semana
** que se corresponde al instante especificado.
** 0=lunes... 6=domingo
*/
unsigned int dia_de_la_semana(time_t tiempo)
{
  char *p = ctime(&tiempo);

  *(p + 3) = '\0';
  if (!strcmp(p, "Mon"))
    return 0;
  if (!strcmp(p, "Tue"))
    return 1;
  if (!strcmp(p, "Wed"))
    return 2;
  if (!strcmp(p, "Thu"))
    return 3;
  if (!strcmp(p, "Fri"))
    return 4;
  if (!strcmp(p, "Sat"))
    return 5;
  return 6;
}

/*
** Devuelve el numero de dias
** de este mes.
*/
unsigned int num_dias_este_mes(time_t tiempo)
{
  int mes;
  char buf[64];
  unsigned int dias;
  unsigned int dias_meses[] =
    { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

  tiempo2str(buf, tiempo);
  mes = (buf[4] - '0') * 10 + (buf[5] - '0');
  dias = dias_meses[mes - 1];
  if (mes == 2) {               /* Febrero */
    buf[4] = '\0';
/*
** Lo que sigue es calendario juliano,
** mientras que lo que se utiliza es
** calendario gregoriano.
**
** Pero son iguales excepto 3 bisiestos
** cada 400 an~os: 1700, 1800, 1900,
** 2100, 2200, 2300...
**
** Curiosamente el 2000 es una excepcion
*/
    if (!(atol(buf) % 4))
      dias++;
  }
  return dias;
}

/*
** Esta rutina da el tiempo en segundos del
** primer segundo del mes cuyo tiempo se indica
*/
time_t dia1_este_mes(time_t tiempo)
{
  char buf[64];
  long fecha;

  tiempo2str(buf, tiempo);
  buf[6] = '\0';
  fecha = atol(buf);
  sprintf(buf, "%lu010000", fecha);
  return str2tiempo(buf);
}

/*
** Esta rutina da el tiempo en segundos del
** primer segundo del mes SIGUIENTE al tiempo
** que se indica.
*/
time_t dia1_siguiente_mes(time_t tiempo)
{
  time_t t, t2;

  tiempo = t = t2 = dia1_este_mes(tiempo);
  do {
    t2 += 25 * (86400ul);       /* Le sumo unos dias, hasta que cambiemos de mes */
    t = dia1_este_mes(t2);
  } while (tiempo == t);

  return t;
}

/*
** clipping_mensual
** Esta rutina devuelve un valor en segundos que
** se corresponde con:
** - El principio del mes en curso
** - El principio del mes anterior si el mes en
**   curso tiene menos de cuatro horas de vida.
*/
time_t clipping_mensual(void)
{
  char buf[CAMPO_LONG];
  unsigned long i;

  tiempo2str(buf, time(NULL));
  if ((buf[6] == '0') && (buf[7] == '1') && (buf[8] == '0') && (buf[9] < '4')) { /* 4 horas */
    buf[6] = '\0';
    i = atol(buf);
    i -= 1;
    if (!(i % 100))
      i -= 88;                  /* Quita uno al ano y pone mes doce */
    sprintf(buf, "%lu010000", i);
  }
  else {
    strcpy(buf + 6, "010000");  /* Desde el principio del mes */
  }
  return str2tiempo(buf);
}

void activa_clipping_mensual(int num_tiempos, char *tiempos[])
{
/*
char buf1[50],buf2[50];
*/

  if (num_tiempos < 1) {        /* No se indican los tiempos */
    tiempo_inicial = clipping_mensual();
  }
  else {
    tiempo_inicial = str2tiempo(tiempos[0]);
    if (num_tiempos > 1)
      tiempo_final = str2tiempo(tiempos[1]);
  }

/*
  tiempo2str(buf1,tiempo_inicial);
  tiempo2str(buf2,tiempo_final);
  printf("Tiempo Inicial: %s\nTiempo Final: %s\n",buf1,buf2);
*/
}
